package imageanalyzer;

import java.awt.BorderLayout;
import java.awt.EventQueue;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;
import javax.swing.UIManager;

public class Main extends JFrame {

	private JLabel lblImmagine;
	private BufferedImage immagine, immagineOriginale;

	public static void main(String[] args) {
		try {
			UIManager.setLookAndFeel("javax.swing.plaf.nimbus.NimbusLookAndFeel");
		} catch (Throwable e) {
			e.printStackTrace();
		}
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					Main frame = new Main();
					frame.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	public Main() {
		setTitle("Image Analyzer");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setIconImage(Toolkit.getDefaultToolkit().getImage(Main.class.getResource("/imageanalyzer/img/imageanalyzer.png")));
		setBounds(100, 100, 512, 288);

		JMenuBar menuBar = new JMenuBar();
		setJMenuBar(menuBar);

		JMenu mnFile = new JMenu("File");
		mnFile.setMnemonic('F');
		menuBar.add(mnFile);

		JMenuItem mntmNuovo = new JMenuItem("Nuovo");
		mntmNuovo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, InputEvent.CTRL_MASK));
		mntmNuovo.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/nuovo.png")));
		mntmNuovo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				nuovo();
			}
		});
		mnFile.add(mntmNuovo);

		JMenuItem mntmApri = new JMenuItem("Apri...");
		mntmApri.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, InputEvent.CTRL_MASK));
		mntmApri.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/apri.png")));
		mntmApri.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				apri();
			}
		});
		mnFile.add(mntmApri);

		JMenuItem mntmSalva = new JMenuItem("Salva...");
		mntmSalva.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.CTRL_MASK));
		mntmSalva.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/salva.png")));
		mntmSalva.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				salva();
			}
		});
		mnFile.add(mntmSalva);

		JSeparator separator = new JSeparator();
		mnFile.add(separator);

		JMenuItem mntmEsci = new JMenuItem("Esci");
		mntmEsci.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F4, InputEvent.ALT_MASK));
		mntmEsci.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				System.exit(0);
			}
		});
		mnFile.add(mntmEsci);

		JMenu mnModifica = new JMenu("Modifica");
		mnModifica.setMnemonic('M');
		menuBar.add(mnModifica);
		
		JMenuItem mntmOriginale = new JMenuItem("Originale");
		mntmOriginale.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				originale();
			}
		});
		mntmOriginale.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/originale.png")));
		mnModifica.add(mntmOriginale);
		
		JMenuItem mntmBiancoENero = new JMenuItem("Bianco e nero");
		mntmBiancoENero.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/bianco-nero.png")));
		mnModifica.add(mntmBiancoENero);
		
		JMenuItem mntmScalaDiGrigi = new JMenuItem("Scala di grigi");
		mntmScalaDiGrigi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				scalaDiGrigi();
			}
		});
		mntmScalaDiGrigi.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/scala-grigi.png")));
		mnModifica.add(mntmScalaDiGrigi);
		
		JMenuItem mntmNegativo = new JMenuItem("Negativo");
		mntmNegativo.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/negativo.png")));
		mnModifica.add(mntmNegativo);
		
		JMenuItem mntmRosso = new JMenuItem("Rosso");
		mntmRosso.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/rosso.png")));
		mnModifica.add(mntmRosso);
		
		JMenuItem mntmVerde = new JMenuItem("Verde");
		mntmVerde.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/verde.png")));
		mnModifica.add(mntmVerde);
		
		JMenuItem mntmBlu = new JMenuItem("Blu");
		mntmBlu.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/blu.png")));
		mnModifica.add(mntmBlu);
		
		JMenu mnAnalizza = new JMenu("Analizza");
		mnAnalizza.setMnemonic('A');
		menuBar.add(mnAnalizza);
		
		JMenuItem mntmIstogramma = new JMenuItem("Istogramma");
		mntmIstogramma.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/istogramma.png")));
		mnAnalizza.add(mntmIstogramma);

		JMenu mnAiuto = new JMenu("?");
		mnAiuto.setMnemonic('?');
		menuBar.add(mnAiuto);
		
		JMenuItem mntmInformazioniSu = new JMenuItem("Informazioni su...");
		mntmInformazioniSu.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/informazioni.png")));
		mntmInformazioniSu.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				informazioniSu();
			}
		});
		mnAiuto.add(mntmInformazioniSu);

		JPanel contentPane = new JPanel();
		contentPane.setLayout(new BorderLayout(0, 0));
		setContentPane(contentPane);

		JToolBar toolBar = new JToolBar();
		contentPane.add(toolBar, "North");

		JButton btnNuovo = new JButton("");
		btnNuovo.setFocusPainted(false);
		btnNuovo.setToolTipText("Nuovo");
		btnNuovo.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/nuovo.png")));
		btnNuovo.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				nuovo();
			}
		});
		toolBar.add(btnNuovo);

		JButton btnApri = new JButton("");
		btnApri.setFocusPainted(false);
		btnApri.setToolTipText("Apri");
		btnApri.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/apri.png")));
		btnApri.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				apri();
			}
		});
		toolBar.add(btnApri);

		JButton btnSalva = new JButton("");
		btnSalva.setFocusPainted(false);
		btnSalva.setToolTipText("Salva");
		btnSalva.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/salva.png")));
		btnSalva.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				salva();
			}
		});
		toolBar.add(btnSalva);
		
		JSeparator separator_1 = new JSeparator();
		separator_1.setOrientation(SwingConstants.VERTICAL);
		toolBar.add(separator_1);
		
		JButton btnOriginale = new JButton("");
		btnOriginale.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				originale();
			}
		});
		btnOriginale.setFocusPainted(false);
		btnOriginale.setToolTipText("Originale");
		btnOriginale.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/originale.png")));
		toolBar.add(btnOriginale);
		
		JButton btnBiancoENero = new JButton("");
		btnBiancoENero.setFocusPainted(false);
		btnBiancoENero.setToolTipText("Bianco e nero");
		btnBiancoENero.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/bianco-nero.png")));
		toolBar.add(btnBiancoENero);
		
		JButton btnScalaDiGrigi = new JButton("");
		btnScalaDiGrigi.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				scalaDiGrigi();
			}
		});
		btnScalaDiGrigi.setFocusPainted(false);
		btnScalaDiGrigi.setToolTipText("Scala di grigi");
		btnScalaDiGrigi.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/scala-grigi.png")));
		toolBar.add(btnScalaDiGrigi);
		
		JButton btnNegativo = new JButton("");
		btnNegativo.setFocusPainted(false);
		btnNegativo.setToolTipText("Negativo");
		btnNegativo.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/negativo.png")));
		toolBar.add(btnNegativo);
		
		JButton btnRosso = new JButton("");
		btnRosso.setFocusPainted(false);
		btnRosso.setToolTipText("Rosso");
		btnRosso.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/rosso.png")));
		toolBar.add(btnRosso);
		
		JButton btnVerde = new JButton("");
		btnVerde.setFocusPainted(false);
		btnVerde.setToolTipText("Verde");
		btnVerde.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/verde.png")));
		toolBar.add(btnVerde);
		
		JButton btnBlu = new JButton("");
		btnBlu.setFocusPainted(false);
		btnBlu.setToolTipText("Blu");
		btnBlu.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/blu.png")));
		toolBar.add(btnBlu);
		
		JSeparator separator_2 = new JSeparator();
		separator_2.setOrientation(SwingConstants.VERTICAL);
		toolBar.add(separator_2);
		
		JButton btnIstogramma = new JButton("");
		btnIstogramma.setFocusPainted(false);
		btnIstogramma.setToolTipText("Istogramma");
		btnIstogramma.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/istogramma.png")));
		toolBar.add(btnIstogramma);
		
		JSeparator separator_3 = new JSeparator();
		separator_3.setOrientation(SwingConstants.VERTICAL);
		toolBar.add(separator_3);

		JButton btnInformazionisu = new JButton("");
		btnInformazionisu.setFocusPainted(false);
		btnInformazionisu.setToolTipText("Informazioni su...");
		btnInformazionisu.setIcon(new ImageIcon(Main.class.getResource("/imageanalyzer/img/informazioni.png")));
		btnInformazionisu.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				informazioniSu();
			}
		});
		toolBar.add(btnInformazionisu);

		JScrollPane scrollPane = new JScrollPane();
		contentPane.add(scrollPane, "Center");
		
		lblImmagine = new JLabel("");
		lblImmagine.setHorizontalAlignment(SwingConstants.CENTER);
		scrollPane.setViewportView(lblImmagine);
	}

	public void nuovo() {
		if (JOptionPane.showConfirmDialog(null, "Se non  stata salvata, l'immagine corrente andr persa.\nDesideri continuare?", "Nuova immagine", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
			immagine = null;
			immagineOriginale = null;
			lblImmagine.setIcon(null);
		}
	}

	public void apri() {
		BufferedImage immagineTmp = BufferedImageIO.read();
		if (immagineTmp != null) {
			immagine = immagineTmp;
			immagineOriginale = immagineTmp;
			lblImmagine.setIcon(new ImageIcon(immagine));
		}
	}

	public void salva() {
		BufferedImageIO.write(immagine);
	}
	
	public void informazioniSu() {
		new InformazioniSu().setVisible(true);
	}

	public void originale() {
		immagine = immagineOriginale;
		lblImmagine.setIcon(new ImageIcon(immagine));
	}

	public void scalaDiGrigi() {
		immagine = Trasforma.inScalaDiGrigi(immagine);
		lblImmagine.setIcon(new ImageIcon(immagine));
	}

}
